local Lib = MissionMinder;
Lib.callbacks= Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local addon = MissionMinder;
addon.listview = {};
local module = addon.listview;
local options = addon.options;
local DEBUG = MissionMinder.debug;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local CLOSE = L["Close"]

function MissionMinder.IsMissionComplete(mission)
	if ( mission.isComplete ~= nil ) then
		return mission.isComplete
	elseif ( mission.state ~= -2 and mission.followers and #mission.followers > 0 ) then
		local missionTimeLeft = C_Garrison.GetFollowerMissionTimeLeftSeconds(mission.followers[1]);
		return missionTimeLeft and missionTimeLeft <= 0
	end

	return false;
end


local function sortMissionsComparator(mission1, mission2)
	if ( options.ListViewEnabled() ) then
		local mission1Complete = MissionMinder.IsMissionComplete(mission1);
		local mission2Complete = MissionMinder.IsMissionComplete(mission2);
		if ( mission1Complete ~= mission2Complete ) then
			return mission1Complete;
		end
	end

	-- Blizzard code below --
	if ( mission1.level ~= mission2.level ) then
		return mission1.level > mission2.level;
	end

	if ( mission1.level == GARRISON_FOLLOWER_MAX_LEVEL ) then	-- mission 2 level is same as 1's at this point
		if ( mission1.iLevel ~= mission2.iLevel ) then
			return mission1.iLevel > mission2.iLevel;
		end
	end

	if ( mission1.durationSeconds ~= mission2.durationSeconds ) then
		return mission1.durationSeconds < mission2.durationSeconds;
	end

	if ( mission1.isRare ~= mission2.isRare ) then
		return mission1.isRare;
	end

	return strcmputf8i(mission1.name, mission2.name) < 0;
end


-- Alter sort to have completed missions at the top
local function sortMissionsCompletedFirst(missionsList)
	table.sort(missionsList, sortMissionsComparator);
end


local function inprogressTooltips(missionInfo, showRewards)
	GameTooltip:SetText(missionInfo.name);
	-- level
	if ( missionInfo.level == GARRISON_FOLLOWER_MAX_LEVEL and missionInfo.iLevel > 0 ) then
		GameTooltip:AddLine(format(GARRISON_MISSION_LEVEL_ITEMLEVEL_TOOLTIP, missionInfo.level, missionInfo.iLevel), 1, 1, 1);
	else
		GameTooltip:AddLine(format(GARRISON_MISSION_LEVEL_TOOLTIP, missionInfo.level), 1, 1, 1);
	end

	-- time
	if ( MissionMinder.IsMissionComplete(missionInfo) ) then
		GameTooltip:AddLine("(" .. COMPLETE .. ")", 0, 1, 0)
	else
		GameTooltip:AddLine(tostring(missionInfo.timeLeft) .. " " .. RED_FONT_COLOR_CODE .. GARRISON_MISSION_IN_PROGRESS .. FONT_COLOR_CODE_CLOSE, 1, 1, 1);
	end

	--[===[@debug@
	GameTooltip:AddLine("MissionID = " .. missionInfo.missionID);
	--@end-debug@]===]

	if ( showRewards ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(REWARDS);
		for id, reward in pairs(missionInfo.rewards) do
			if (reward.quality) then
				GameTooltip:AddLine(ITEM_QUALITY_COLORS[reward.quality + 1].hex..reward.title..FONT_COLOR_CODE_CLOSE);
			elseif (reward.itemID) then
				local itemName, _, itemRarity, _, _, _, _, _, _, itemTexture = GetItemInfo(reward.itemID);
				if itemName then
					GameTooltip:AddLine(ITEM_QUALITY_COLORS[itemRarity].hex..itemName..FONT_COLOR_CODE_CLOSE);
				end
			elseif (reward.followerXP) then
				GameTooltip:AddLine(reward.title, 1, 1, 1);
			else
				GameTooltip:AddLine(reward.title, 1, 1, 1);
			end
		end
	end

	if (missionInfo.followers ~= nil) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(GARRISON_FOLLOWERS);
		for i=1, #(missionInfo.followers) do
			GameTooltip:AddLine(C_Garrison.GetFollowerName(missionInfo.followers[i]), 1, 1, 1);
		end
	end
end

local function showInProgressMissionList(...)
	if ( options.ListViewEnabled() ) then
		PlaySound("UI_Garrison_CommandTable_ViewMissionReport");
		GarrisonMissionList_SetTab(GarrisonMissionFrame.MissionTab.MissionList.Tab2);
		GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog:Hide();
	else
		addon.GarrisonMissionFrame_ShowCompleteMissions(...);
	end
end

local function GetInProgressAndCompletedMissions(missions)
	C_Garrison.GetInProgressMissions(missions);

	if ( options.ListViewEnabled() ) then
		GarrisonMissionFrame.MissionComplete.completeMissions = C_Garrison.GetCompleteMissions();
		local completed = GarrisonMissionFrame.MissionComplete.completeMissions;

		-- Add completed mission to list
		for index = 1, #completed do
			local mission = completed[index];
			if ( mission.state == 0 ) then -- COMPLETED
				mission.inProgress = true;
				mission.timeLeft = ""; -- set to non-nil to avoid nil error in blizzard code
				missions[#missions + 1] = mission;
			end
		end
	end

	Garrison_SortMissions(missions);
end


-- Pretty much the same code as the standard UI, except that this calls GetInProgressAndCompletedMissions
local function missionListUpdate_IncludeCompletedMissions(...)
	local self = GarrisonMissionFrame.MissionTab.MissionList;
	GetInProgressAndCompletedMissions(self.inProgressMissions);
	C_Garrison.GetAvailableMissions(self.availableMissions);
	Garrison_SortMissions(self.availableMissions);
	self.Tab1:SetText(AVAILABLE.." - "..#self.availableMissions)
	self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)
	if ( #self.inProgressMissions > 0 ) then
		self.Tab2.Left:SetDesaturated(false);
		self.Tab2.Right:SetDesaturated(false);
		self.Tab2.Middle:SetDesaturated(false);
		self.Tab2.Text:SetTextColor(1, 1, 1);
		self.Tab2:SetEnabled(true);
	else
		self.Tab2.Left:SetDesaturated(true);
		self.Tab2.Right:SetDesaturated(true);
		self.Tab2.Middle:SetDesaturated(true);
		self.Tab2.Text:SetTextColor(0.5, 0.5, 0.5);
		self.Tab2:SetEnabled(false);
	end
	GarrisonMissionList_Update();
end

local function missionList_OnUpdate(now, self)
	if (self.showInProgress) then
		GetInProgressAndCompletedMissions(self.inProgressMissions);
		self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)
		GarrisonMissionList_Update();
	else
		local now = GetTime();
		for i = 1, #self.availableMissions do
			if ( self.availableMissions[i].offerEndTime and self.availableMissions[i].offerEndTime <= now ) then
				GarrisonMissionList_UpdateMissions();
				break;
			end
		end
	end
end

local nextMissionListUpdate = nil;
local function missionListOnUpdate_IncludeCompletedMissions(self)
	local now = GetTime();
	if ( not nextMissionListUpdate or now > nextMissionListUpdate ) then
		nextMissionListUpdate = now + 0.2 -- Update every 0.2 seconds
		missionList_OnUpdate(now, self)
	end
end

function Lib.findCompleteMission(missionID)
	for r = 1, 2 do
		if ( r == 2 ) then -- Seconde iteration, update complete missions and try again
			GarrisonMissionFrame.MissionComplete.completeMissions = C_Garrison.GetCompleteMissions();
		end

		local missions = GarrisonMissionFrame.MissionComplete.completeMissions or {}
		for i = 1, #missions do
			if ( missionID == missions[i].missionID ) then
				return missions[i], i
			end
		end
	end

	DEBUG("findCompleteMissionIndex(%d) => nil", missionID)
	return nil
end


-- When user clicks on a mission from the mission list (either available or in progress)
local function viewMission(...)
	local self, button = ...

	if ( not options.ListViewEnabled() ) then
		return;
	end

	-- Ignore CHATLINK attempts and non-inProgress missions (handled properly already)
	if ( IsModifiedClick("CHATLINK") or not self.info.inProgress ) then
		return;
	end

	local mission = self.info;
	if ( MissionMinder.IsMissionComplete(mission) ) then -- Mission must be completed
		if ( options.QuickLoot() ) then
			MissionMinder.quickloot.lootMission(mission);
		else
			-- This would be part of the OnClick event of the In Progress tab
			GarrisonMissionFrame.FollowerTab:Hide();
			GarrisonMissionFrame.FollowerList:Hide();
			HelpPlate_Hide();

			GarrisonMissionFrame.MissionComplete:Show();
			GarrisonMissionFrame.MissionCompleteBackground:Show();

			local self = GarrisonMissionFrame.MissionComplete;
			self.currentIndex = select(2, Lib.findCompleteMission(mission.missionID))

			if ( self.currentIndex ) then
				MissionCompletePreload_Cancel(); -- Cancel any previous preload that is still running
				MissionCompletePreload_LoadMission(mission.missionID); -- Start preload
				GarrisonMissionComplete_Initialize(self.completeMissions, self.currentIndex);
			end
		end
	end
end



local function highlightCompletedMissions(...)
	-- Hide overlay on completed missions and highlight them as finished
	local self = GarrisonMissionFrame.MissionTab.MissionList;
	local buttons = self.listScroll.buttons;

	for _,button in pairs(buttons) do
		local mission = button.info
		if ( mission ) then

			if ( self.showInProgress ) then
				local _, _, _, successChance = C_Garrison.GetPartyMissionInfo(mission.missionID); -- Success chance may be nil if an update occurs while the mission is removed
				if ( successChance ) then
					button.SuccessChance:SetFormattedText(PERCENTAGE_STRING .. " " .. GARRISON_MISSION_CHANCE, successChance);
					button.SuccessChance:Show();
				else
					button.SuccessChance:SetText(nil);
					button.SuccessChance:Hide();
				end

				MissionMinder_UpdatePartyMembers(button, mission.followers)
			end

			if( options.ListViewEnabled() and self.showInProgress and MissionMinder.IsMissionComplete(mission)) then
				button.Overlay:Hide();

				button.LocBG:SetVertexColor(0.3921, 1, 1, 1); -- green blue
				button.Summary:SetText("(" .. COMPLETE .. ")");
				button.Summary:SetTextColor(0, 1, 0);
			else
				button.Summary:SetTextColor(1, 1, 1);
				button.LocBG:SetVertexColor(1, 1, 1, 1);
			end

			-- Summary should always show to the right of title
			button.Summary:ClearAllPoints();
			button.Summary:SetPoint("BOTTOMLEFT", button.Title, "BOTTOMRIGHT", 8, 0);

			-- Truncate title if too long
			button.Title:SetWidth(0)
			local maxButtonWidth = 655 - (mission.numRewards * 65) - button.Summary:GetWidth() - 8
			if ( button.Title:GetWidth() > maxButtonWidth ) then
				button.Title:SetWidth(maxButtonWidth)
			end

		end
	end
end

local function showMissionCompleteFrame(...)
	-- Do not popup mission complete dialog if the "In Progress" mission tab is already selected
	local self = GarrisonMissionFrame;
	if ( options.ListViewEnabled() ) then
		if( self:IsShown() and self.MissionTab.MissionList.showInProgress) then
			self.MissionComplete.completeMissions = C_Garrison.GetCompleteMissions();
			return;
		end
	end

	addon.GarrisonMissionFrame_CheckCompleteMissions(...)
end


local function closeMissionCompleteFrame(...)
	if ( options.ListViewEnabled() ) then
		GarrisonMissionFrame_HideCompleteMissions(true);
	else
		addon.GarrisonMissionCompleteNextButton_OnClick(...)
	end
end


local function updateNextMissionButtonText(...)
	local nextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton;

	if ( options.ListViewEnabled() ) then
		nextMissionButton:SetText(CLOSE);
	else
		nextMissionButton:SetText(NEXT);
	end
end


local function selectAvailableMissions(...)
	GarrisonMissionList_SetTab(GarrisonMissionFrame.MissionTab.MissionList.Tab1);
end

local function refreshCompletedMissions()
	if ( options.ListViewEnabled() ) then
		local self = GarrisonMissionFrame.MissionComplete;
		self.completeMissions = C_Garrison.GetCompleteMissions();

		GetInProgressAndCompletedMissions(GarrisonMissionFrame.MissionTab.MissionList.inProgressMissions);
		GarrisonMissionList_Update();
	end
end

local function refreshCompletedMissionsForQuickLoot()
	if ( options.QuickLoot() ) then
		refreshCompletedMissions()
	end
end

local function setMissionButtonPartyMember(frame, followerID)
	if ( followerID ) then
		local info = C_Garrison.GetFollowerInfo(followerID)
		frame.info = info

		local color = ITEM_QUALITY_COLORS[info.quality];
		frame.PortraitRingQuality:SetVertexColor(color.r, color.g, color.b);
		if ( info.displayID ) then
			GarrisonFollowerPortrait_Set(frame.Portrait, info.portraitIconID);
		end
	else
		frame.info = nil;
		frame.PortraitRingQuality:SetVertexColor(0.3, 0.3, 0.3);
		GarrisonFollowerPortrait_Set(frame.Portrait, nil);
	end
end

function MissionMinder_UpdatePartyMembers(button, followers, followerCount)
	if ( followers and not followerCount) then
		followerCount = #followers
	end

	if(followers and followerCount > 0) then
		button.Title:ClearAllPoints();
		button.Title:SetPoint("TOPLEFT", 165, -5);

		for i=1, followerCount do
			local frame = button.Party[i]
			if ( not frame ) then
				frame = CreateFrame("Frame", nil, button, "MissionMinder_GarrisonFollowerPortrait")
				frame:SetPoint("LEFT", button.Party[i-1], "RIGHT", 10, 0);
				tinsert(button.Party, frame)
			end

			setMissionButtonPartyMember(frame, followers[i])
			frame:Show()
		end
	else
		followerCount = 0
		button.Title:ClearAllPoints();
		button.Title:SetPoint("LEFT", 165, 0);
	end

	for i=followerCount+1,#button.Party do
		local frame = button.Party[i]
		setMissionButtonPartyMember(frame, showItemLevel, nil)
		frame:Hide()
	end
end

function MissionMinder_MissionFollowerFrame_OnEnter(self)
	local follower = self.info
	local mission = self:GetParent().info
	if not follower or not mission then
		return;
	end

	GarrisonFollowerTooltip:ClearAllPoints();
	GarrisonFollowerTooltip:SetPoint("TOPLEFT", self, "BOTTOMRIGHT");
	GarrisonFollowerTooltip_Show(follower.garrFollowerID,
		follower.isCollected,
		C_Garrison.GetFollowerQuality(follower.followerID),
		C_Garrison.GetFollowerLevel(follower.followerID),
		C_Garrison.GetFollowerXP(follower.followerID),
		C_Garrison.GetFollowerLevelXP(follower.followerID),
		C_Garrison.GetFollowerItemLevelAverage(follower.followerID),
		C_Garrison.GetFollowerAbilityAtIndex(follower.followerID, 1),
		C_Garrison.GetFollowerAbilityAtIndex(follower.followerID, 2),
		C_Garrison.GetFollowerAbilityAtIndex(follower.followerID, 3),
		C_Garrison.GetFollowerAbilityAtIndex(follower.followerID, 4),
		C_Garrison.GetFollowerTraitAtIndex(follower.followerID, 1),
		C_Garrison.GetFollowerTraitAtIndex(follower.followerID, 2),
		C_Garrison.GetFollowerTraitAtIndex(follower.followerID, 3),
		C_Garrison.GetFollowerTraitAtIndex(follower.followerID, 4),
		true,
		C_Garrison.GetFollowerBiasForMission(mission.missionID, follower.followerID) < 0.0
		);
end

local function initialize()
--function module.initialize()
	DEBUG("   |cffffffffListView|r initialized")

	local MissionList = GarrisonMissionFrame.MissionTab.MissionList
	-- "View Completed Missions" button will open "In Progress" tab instead of "CompleteMission" frame
	addon.GarrisonMissionFrame_ShowCompleteMissions = GarrisonMissionFrame_ShowCompleteMissions;
	local ViewButton = MissionList.CompleteDialog.BorderFrame.ViewButton;
	ViewButton:SetScript("OnClick", showInProgressMissionList);

	-- Completed Missions that haven't been looted are not considered "in progress" anymore
	--and need to be added to the list
	GarrisonMissionList_UpdateMissions = missionListUpdate_IncludeCompletedMissions;
	GarrisonMissionList_OnUpdate = missionListOnUpdate_IncludeCompletedMissions;
	MissionList:SetScript("OnUpdate", GarrisonMissionList_OnUpdate)

	-- Sort In Progress missions by completed first
	Garrison_SortMissions = sortMissionsCompletedFirst;

	-- Highlight completed missions
	hooksecurefunc("GarrisonMissionList_Update", highlightCompletedMissions);
	GarrisonMissionFrame.MissionTab.MissionList.listScroll.update = GarrisonMissionList_Update; -- Set the update method to the new hooked method

	-- Clicking a completed mission will view it
	local missionButtons = GarrisonMissionFrame.MissionTab.MissionList.listScroll.buttons;
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		button:HookScript("OnClick", viewMission);
	end

	-- Show special tooltips for completed missions
	GarrisonMissionButton_SetInProgressTooltip = inprogressTooltips

	-- Create SuccessChance text
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		local successChance = button:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
		successChance:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -17);
		successChance:SetTextColor(0, 1, 0);
		successChance:SetSize(115, 15)
		button.SuccessChance = successChance;
	end

	-- Create Party member frames
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		button.Party = {}
		local partyMember = CreateFrame("Frame", nil, button, "MissionMinder_GarrisonFollowerPortrait")
		partyMember:SetPoint("LEFT", button.SuccessChance, "RIGHT", 20, 0);
		tinsert(button.Party, partyMember)
	end

	-- rewrite "Next" mission button as "Close"
	local MissionComplete = GarrisonMissionFrame.MissionComplete;
	MissionComplete:HookScript("OnShow", updateNextMissionButtonText);

	-- Clicking "Next" misison button closes the mission complete frame
	addon.GarrisonMissionCompleteNextButton_OnClick = GarrisonMissionCompleteNextButton_OnClick
	local NextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton;
	NextMissionButton:SetScript("OnClick", closeMissionCompleteFrame);

	-- Refresh completed missions list on GarrisonMissionFrame.MissionComplete:Hide
	local events = MissionMinder.events;
	events:RegisterEvent("listview", "GARRISON_MISSION_COMPLETE_RESPONSE", refreshCompletedMissionsForQuickLoot);
	events:RegisterEvent("listview", "GARRISON_MISSION_BONUS_ROLL_COMPLETE", refreshCompletedMissionsForQuickLoot);
	GarrisonMissionFrame.MissionComplete:HookScript("OnHide", refreshCompletedMissions);

	-- Closing GarrisonMissionFrame should select the "Available Missions" tab. This will cause the next open to show
	--the completed missions dialog again (if applicable)
	GarrisonMissionFrame:HookScript("OnHide", selectAvailableMissions);

	-- Mission finished event should not show complete dialog if "In Progress" tab already selected
	-- Currently uses a pre-hook, with more work might be able to remove the pre-hook requirement and use secure post-hook
	addon.GarrisonMissionFrame_CheckCompleteMissions = GarrisonMissionFrame_CheckCompleteMissions;
	GarrisonMissionFrame_CheckCompleteMissions = showMissionCompleteFrame;
end

local events = Lib.events
events:RegisterEvent("listview", "ADDON_LOADED", initialize)
