local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local GARRISON_PLANNER_SENDING_MISSIONS = L["Sending missions..."]
local GARRISON_PLANNER_PROGRESS = L["%d of %d"];

local STARTING_MISSIONS = {}
module.STARTING_MISSIONS = STARTING_MISSIONS


local function animateSend(self, index)
	local PlannerTab = GarrisonMissionFrame.PlannerTab
	local offset = HybridScrollFrame_GetOffset(PlannerTab.listScroll);
	local buttons = PlannerTab.listScroll.buttons

	PlannerTab.StartingOffscreenMissions:Hide()
	if ( index <= offset ) then
		local GlowBox = PlannerTab.StartingOffscreenMissions.GlowBox
		GlowBox:ClearAllPoints()
		GlowBox:SetPoint("TOP",  PlannerTab.listScroll, 0, -5)
		GlowBox.ArrowDown:Hide()
		GlowBox.ArrowGlowDown:Hide()
		GlowBox.ArrowUp:Show()
		GlowBox.ArrowGlowUp:Show()
		GlowBox.BigText:SetText(GARRISON_PLANNER_SENDING_MISSIONS)
		GlowBox.SmallText:SetFormattedText(GARRISON_PLANNER_PROGRESS, index, #self.missions)
		GlowBox:SetHeight(12 + GlowBox.BigText:GetHeight() + 12 + GlowBox.SmallText:GetHeight() + 12)
		PlannerTab.StartingOffscreenMissions:Show()
	elseif ( index >= offset + #buttons ) then
		local GlowBox = PlannerTab.StartingOffscreenMissions.GlowBox
		GlowBox:ClearAllPoints()
		GlowBox:SetPoint("BOTTOM",  PlannerTab.listScroll, 0, 5)
		GlowBox.ArrowUp:Hide()
		GlowBox.ArrowGlowUp:Hide()
		GlowBox.ArrowDown:Show()
		GlowBox.ArrowGlowDown:Show()
		GlowBox.BigText:SetText(GARRISON_PLANNER_SENDING_MISSIONS)
		GlowBox.SmallText:SetFormattedText(GARRISON_PLANNER_PROGRESS, index, #self.missions)
		GlowBox:SetHeight(12 + GlowBox.BigText:GetHeight() + 12 + GlowBox.SmallText:GetHeight() + 12)
		PlannerTab.StartingOffscreenMissions:Show()
	else
		buttons[index - offset].SendMission:Play()
	end
end

local function startNextMission(self)
	if ( self.index < #self.missions ) then
		self.index = self.index + 1
		module.startMission(self.missions[self.index])
		animateSend(self, self.index)
	end

	if ( self.index >= #self.missions ) then
		self.timerTask:Cancel()
		self.timerTask = nil

		C_Timer.After(0.5, function()
			module.setState(module.PLANNER)
		end)
	end
end

function STARTING_MISSIONS:New(missions)
	local self = {
		["name"] = "STARTING_MISSIONS",
		["missions"] = missions,
		["index"] = 0,
		["timerTask"] = nil
	}

	function self:initialize()
		module.hideStatusOverlay()

		-- Show looping Starting Missions animation

		-- Prepare SendMisssions animation
		local buttons = GarrisonMissionFrame.PlannerTab.listScroll.buttons
		for i = 1, #buttons do
			buttons[i].SendMission:Stop()
			buttons[i].Overlay:Hide()
		end

		-- Update mission list
		GarrisonMissionFrame.PlannerTab.listScroll.scrollBar:SetValue(0)
		GarrisonMissionPlanner_DeselectMission()

		GarrisonMissionFrame.PlannerTab.StartingOffscreenMissions:Hide()

		self.timerTask = C_Timer.NewTicker(0.5, function()
			startNextMission(self)
		end, #missions);
	end

	function self:destroy()
		-- Stop looping Starting Missions animation

		-- Cleanup SendMisssions animation
		local buttons = GarrisonMissionFrame.PlannerTab.listScroll.buttons
		for i = 1, #buttons do
			buttons[i].SendMission:Stop()
			buttons[i].Overlay:Hide()
		end

		GarrisonMissionFrame.PlannerTab.StartingOffscreenMissions:Hide()
	end

	function self:updateMissions()
		local plannerTab = GarrisonMissionFrame.PlannerTab;
		local missions = self.missions
		local numMissions = #missions;

		local scrollFrame = plannerTab.listScroll;
		local offset = HybridScrollFrame_GetOffset(scrollFrame);
		local buttons = scrollFrame.buttons;
		local numButtons = #buttons;

		module.hideStatusOverlay()

		for i = 1, numButtons do
			local button = buttons[i];
			local index = offset + i;

			module.setMissionButton(button, missions, index)
			module.markForUpdate(button)

			button.SendMission:Stop()
			button.Overlay:Hide()
			if (self.index >= index ) then
				button.Overlay:Show()
				button.Overlay:SetAlpha(0.8)
			end
		end

		-- show breadcrumb for missions sent off screen
		if ( self.index < offset ) then

		elseif ( self.index >= offset + numButtons ) then

		end

		-- Animate sent missions based on current index
		-- If current index is off the screen then we show breadcrumb

		local totalHeight = numMissions * scrollFrame.buttonHeight;
		local displayedHeight = numButtons * scrollFrame.buttonHeight;
		HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);
	end

	function self:buttonEnter()
	end

	function self:buttonClick()
	end

	function self:buttonDoubleClick()
	end

	return self
end
