local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local OPTIONS = Lib.options;
local DEBUG = Lib.debug;

MissionMinder.multiplanner = {};
local module = Lib.multiplanner;

GARRISON_PLANNER = L["Planner"];
GARRISON_PLANNER_START_MISSIONS = L["Start Missions"];
GARRISON_PLANNER_TOO_MANY_ACTIVE_FOLLOWERS = L["Too many active followers"];
GARRISON_PLANNER_NOT_ENOUGH_FOLLOWERS_FOR_CURRENT_MISSIONS = L["Not enough available followers for current missions"];
GARRISON_PLANNER_NO_MISSION_SELECTED = L["No mission has been selected."];

local MULTIPLANNER_TAB = 3

local DEFAULT_MISSION_COST_COLOR = { 1, 1, 1, 1 }
local NOT_ENOUGH_MISSION_COST_COLOR = { 1, 0, 0, 1 }

local missionsReadyToSend = 0;
local readyMissionCost = 0
local missionFollowerCounts = {};

local startMissionTask = nil;
local missionsToSend = {};
local startMissionIndex = 0;

module.state = nil

function module.getState()
	return module.state or {}
end

function module.setState(state, ...)
	if ( module.state ) then
		module.state:destroy()
	end
	module.state = state:New(...)
	module.state:initialize()
	DEBUG("State changed to %s", module.state.name)
end

function module.setMissionButton(button, missions, index)
	if ( index <= #missions) then
		local mission = missions[index];
		button.id = index;
		button.info = mission;
		button:Show();
	else
		button.id = nil
		button.info = nil
		button:Hide();
	end
end

function module.hideStatusOverlay()
	local self = GarrisonMissionFrame.PlannerTab;
	self.Overlay:Hide();
end

function module.showStatusOverlay(text, ...)
	local self = GarrisonMissionFrame.PlannerTab;
	self.Overlay.Status:SetText(string.format(text, ...));
	self.Overlay:Show();
end

function PlannerMissionList_Update()
	local state = module.getState()
	if ( state.updateMissions ) then
		state:updateMissions()
	end
end


local function updateStartMissionsButton()
	local MissionCost = GarrisonMissionFrame.PlannerTab.MissionCost.Cost
	MissionCost:SetText(readyMissionCost)
	MissionCost:SetTextColor(unpack(DEFAULT_MISSION_COST_COLOR))

	local _, garrisonResources = GetCurrencyInfo(GARRISON_CURRENCY)

	local errorString = nil;
	if ( missionsReadyToSend == 0 ) then
		errorString = GARRISON_PARTY_NOT_FULL_TOOLTIP
	elseif ( garrisonResources < readyMissionCost ) then
		errorString = GARRISON_NOT_ENOUGH_MATERIALS_TOOLTIP
		MissionCost:SetTextColor(unpack(NOT_ENOUGH_MISSION_COST_COLOR))
	end

	local StartMissions = GarrisonMissionFrame.PlannerTab.StartMissions
	if ( errorString ) then
		StartMissions:SetText(GARRISON_PLANNER_START_MISSIONS)
		StartMissions.tooltip = errorString
		StartMissions.Flash:Hide()
		StartMissions.FlashAnim:Stop()
		StartMissions:Disable()
	else
		StartMissions:SetFormattedText("%s (%d)", GARRISON_PLANNER_START_MISSIONS, missionsReadyToSend)
		StartMissions.tooltip = nil
		StartMissions.Flash:Show()
		StartMissions.FlashAnim:Play()
		StartMissions:Enable()
	end
end

local function increaseReadyMissions(mission)
	missionsReadyToSend = missionsReadyToSend + 1
	readyMissionCost = readyMissionCost + mission.cost
	updateStartMissionsButton()
end

local function decreaseReadyMissions(mission)
	missionsReadyToSend = missionsReadyToSend - 1
	readyMissionCost = readyMissionCost - mission.cost
	updateStartMissionsButton()
end

function module.startMission(mission)
	if ( mission ) then
		DEBUG("Sending mission (%d) \"%s\"", mission.missionID, mission.name);
		C_Garrison.StartMission(mission.missionID);
		PlaySound("UI_Garrison_CommandTable_MissionStart");

		decreaseReadyMissions(mission);
	end
end

function GarrisonMissionPlannerStartMissions_OnClick(self, button)
	local state = module.getState()
	if ( state.name == "PLANNER" ) then
		local missionsToSend = {}
		local missions = C_Garrison.GetAvailableMissions();
		for i = 1, #missions do
			local mission = missions[i];
			if ( #mission.followers == mission.numFollowers ) then
				tinsert(missionsToSend, mission);
			end
		end

		if ( #missionsToSend > 0 ) then
			module.setState(module.STARTING_MISSIONS, missionsToSend)
		end
	end
end

local function clearAllParties()
	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i];
		module.clearFollowers(mission);
	end
end


function module.updateReadyMissionCount()
	missionsReadyToSend = 0;
	readyMissionCost = 0
	missionFollowerCounts = {};
	updateStartMissionsButton()

	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i]
		missionFollowerCounts[mission.missionID] = mission.numFollowers - #mission.followers
		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			increaseReadyMissions(mission)
		end
	end
end


function GarrisonMissionPlanner_ClearFollower(frame, updateValues)
	local followerID = frame.info and frame.info.followerID or nil;
	local mission = frame:GetParent().info;

	frame.info = nil;

	if (followerID) then
		C_Garrison.RemoveFollowerFromMission(mission.missionID, followerID)
		Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, frame:GetID(), "MultiPlanner")

		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			decreaseReadyMissions(mission)
		end
		missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] + 1

		if ( updateValues ) then
			PlaySound("UI_Garrison_CommandTable_UnassignFollower");

			-- update follower list
			GarrisonMissionFrame.FollowerList.dirtyList = true
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
		end
	end

	module.markForUpdate(frame:GetParent());
end

local function dirtyDetailsPane()
	GarrisonMissionFrame.PlannerTab.DetailsPane.dirty = true
end

local function GarrisonMissionPlannerDetailsFrame_CheckCounter(self, counterID)
	for e = 1, #self.Enemies do
		local enemy = self.Enemies[e]
		for m = 1, #enemy.Mechanics do
			local mechanic = enemy.Mechanics[m]
			if ( counterID == mechanic.mechanicID and not mechanic.hasCounter ) then
				mechanic.hasCounter = true;
				mechanic.Check:Show();
				return;
			end
		end
	end
end

local function updateDetailsPaneForParty()
	local mission = GarrisonMissionFrame.MissionTab.MissionPage.missionInfo
	if ( not mission ) then
		return
	end

	local missionID = mission.missionID
	local self = GarrisonMissionFrame.PlannerTab.DetailsPane;

	-- MECHANICS
	for e = 1, #self.Enemies do
		local enemy = self.Enemies[e]
		for m = 1, #enemy.Mechanics do
			enemy.Mechanics[m].hasCounter = nil;
			enemy.Mechanics[m].Check:Hide();
		end
	end
	local party = module.parties[missionID] ;
	if ( party ) then
		for i = 1, #party do
			local followerID = party[i]
			if ( followerID and C_Garrison.GetFollowerBiasForMission(missionID, followerID) > -1 ) then
				local abilities = C_Garrison.GetFollowerAbilities(followerID)
				for a = 1, #abilities do
					local ability = abilities[a];
					for counterID, counterInfo in pairs(ability.counters) do
						GarrisonMissionPlannerDetailsFrame_CheckCounter(self, counterID);
					end
				end
			end
		end
	end
end

local rateBack = 0.1;
local rateMid = 0.3;
local rateFore = 0.8;
function GarrisonPlanner_DetailsPane_OnUpdate(self, elapsed)
	local changeBack = rateBack/100 * elapsed;
	local changeMid = rateMid/100 * elapsed;
	local changeFore = rateFore/100 * elapsed;

	local backL, _, _, _, backR = self.LocBack:GetTexCoord();
	local midL, _, _, _, midR = self.LocMid:GetTexCoord();
	local foreL, _, _, _, foreR = self.LocFore:GetTexCoord();

	backL = backL + changeBack;
	backR = backR + changeBack;
	midL = midL + changeMid;
	midR = midR + changeMid;
	foreL = foreL + changeFore;
	foreR = foreR + changeFore;

	if (backL >= 1) then
		backL = backL - 1;
		backR = backR - 1;
	end
	if (midL >= 1) then
		midL = midL - 1;
		midR = midR - 1;
	end
	if (foreL >= 1) then
		foreL = foreL - 1;
		foreR = foreR - 1;
	end

	self.LocBack:SetTexCoord(backL, backR, 0, 1);
	self.LocMid:SetTexCoord (midL, midR, 0, 1);
	self.LocFore:SetTexCoord(foreL, foreR, 0, 1);

	if ( self.dirty ) then
		updateDetailsPaneForParty()
		self.dirty = nil
	end
end

local function updateDetailsPaneOnFollowerChange(event, missionID)
	local selectedMission = GarrisonMissionFrame.MissionTab.MissionPage.missionInfo
	if ( selectedMission and selectedMission.missionID == missionID ) then
		dirtyDetailsPane()
	end
end

local function updateDetailsPane(event, missionID)
	local self = GarrisonMissionFrame.PlannerTab.DetailsPane;

	local numVisibleEnemies = 0;
	if ( missionID ) then
		local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(missionID);

		if ( locPrefix ) then
			self.LocBack:SetAtlas("_"..locPrefix.."-Back", true);
			self.LocMid:SetAtlas ("_"..locPrefix.."-Mid", true);
			self.LocFore:SetAtlas("_"..locPrefix.."-Fore", true);
		end


		self.EmptyString:Hide();
		for i=1, #enemies do
			local Frame = self.Enemies[i];
			if ( not Frame ) then
				break;
			end
			numVisibleEnemies = numVisibleEnemies + 1;
			local enemy = enemies[i];
			Frame.Name:SetText(enemy.name);
			GarrisonEnemyPortait_Set(Frame.PortraitFrame.Portrait, enemy.portraitFileDataID);
			local numMechs = 0;
			for id, mechanic in pairs(enemy.mechanics) do
				numMechs = numMechs + 1;
				if (not Frame.Mechanics[numMechs]) then
					Frame.Mechanics[numMechs] = CreateFrame("Button", nil, Frame, "GarrisonMissionEnemyLargeMechanicTemplate");
					Frame.Mechanics[numMechs]:SetPoint("LEFT", Frame.Mechanics[numMechs-1], "RIGHT", 16, 0);
					Frame.Mechanics[numMechs]:SetScript("OnClick", MissionMinder_FilterFollowersForCounter)
				end
				local Mechanic = Frame.Mechanics[numMechs];
				Mechanic.info = mechanic;
				Mechanic.Icon:SetTexture(mechanic.icon);
				Mechanic.mechanicID = id;
				Mechanic:Show();
			end
			Frame.Mechanics[1]:SetPoint("BOTTOM", (numMechs - 1) * -22, -16);
			for j=(numMechs + 1), #Frame.Mechanics do
				Frame.Mechanics[j]:Hide();
				Frame.Mechanics[j].mechanicID = nil;
				Frame.Mechanics[j].info = nil;
			end
			if ( numMechs > 1 ) then
				Frame.PortraitFrame.Elite:Show();
			else
				Frame.PortraitFrame.Elite:Hide();
			end
			Frame:Show();

			local numEnemies = math.min(#enemies, #self.Enemies)
			if ( numVisibleEnemies == 1 ) then
				self.Enemy1:SetPoint("TOPLEFT", 251, -30);
			elseif ( numVisibleEnemies == 2 ) then
				self.Enemy1:SetPoint("TOPLEFT", 165, -30);
			else
				self.Enemy1:SetPoint("TOPLEFT", 78, -30);
			end
		end
	else
		self.LocBack:SetAtlas("_GarrMissionLocation-TannanJungle-Back", true);
		self.LocMid:SetAtlas ("_GarrMissionLocation-TannanJungle-Mid", true);
		self.LocFore:SetAtlas("_GarrMissionLocation-TannanJungle-Fore", true);
		self.EmptyString:Show();
	end

	for i = numVisibleEnemies + 1, #self.Enemies do
		self.Enemies[i]:Hide();
	end

	dirtyDetailsPane()
end

function module.IsMissionSelected(mission)
	local selectedMission = GarrisonMissionFrame.MissionTab.MissionPage.missionInfo
	return selectedMission and mission and selectedMission.missionID == mission.missionID
end

function GarrisonMissionPlanner_DeselectMission()
	GarrisonMissionFrame.MissionTab.MissionPage.missionInfo = nil
	Lib.callbacks:Fire("MISSION_SELECTED", nil)
end

function GarrisonMissionPlanner_SelectMission(mission)
	if ( mission ) then
		GarrisonMissionFrame.MissionTab.MissionPage.missionInfo = mission
		Lib.callbacks:Fire("MISSION_SELECTED", mission.missionID)
	end
end


function GarrisonMissionPlanner_AddFollower(followerID)
	local mission = GarrisonMissionFrame.MissionTab.MissionPage.missionInfo
	if ( mission and followerID ) then
		if ( not C_Garrison.AddFollowerToMission(mission.missionID, followerID) ) then
			return;
		end
		Lib.callbacks:Fire("FOLLOWER_ADDED_TO_MISSION", mission.missionID, nil, followerID, "MultiPlanner")

		missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] - 1
		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			increaseReadyMissions(mission)
		end

		PlannerMissionList_Update()
	end
end

function GarrisonMissionPlanner_SetFollower(frame, follower)
	if (frame.info) then
		GarrisonMissionPlanner_ClearFollower(frame);
	end

	local mission = frame:GetParent().info;
	if ( not C_Garrison.AddFollowerToMission(mission.missionID, follower.followerID) ) then
		return;
	end
	Lib.callbacks:Fire("FOLLOWER_ADDED_TO_MISSION", mission.missionID, frame:GetID(), follower.followerID, "MultiPlanner")

	frame.info = follower;
	PlaySound("UI_Garrison_CommandTable_AssignFollower");

	missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] - 1
	if ( missionFollowerCounts[mission.missionID] == 0 ) then
		increaseReadyMissions(mission)
	end

	-- force button to update immediately
	module.markForUpdate(frame:GetParent());
end

local function updateFollowerListForMission(event, missionID)
	if ( missionID ) then
		GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(missionID)
		GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(missionID)
	else
		GarrisonMissionFrame.followerCounters = nil
		GarrisonMissionFrame.followerTraits = nil
	end

	if ( GarrisonMissionFrame.FollowerList.followers ) then
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList)
	end
end


local function selectTab(id)
	if( id == MULTIPLANNER_TAB ) then -- Planner
		PanelTemplates_SetTab(GarrisonMissionFrame, id);

		GarrisonMissionFrame.MissionTab:Hide();
		GarrisonMissionFrame.FollowerTab:Hide();
		GarrisonMissionFrame.PlannerTab:Show()

		if ( GarrisonMissionFrame.FollowerList:IsShown() ) then
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
		else
			GarrisonMissionFrame.FollowerList:Show();
		end
		GarrisonMissionFrame.TitleText:SetText(L["Garrison Mission Planner"]);

		if ( UIDropDownMenu_GetCurrentDropDown() == GarrisonFollowerOptionDropDown ) then
			CloseDropDownMenus();
		end
	else -- Mission or Follower
		GarrisonMissionFrame.PlannerTab:Hide()
		module.GarrisonMissionFrame_SelectTab(id)
	end
end

local function selectMissionsTab()
	if (PanelTemplates_GetSelectedTab(GarrisonMissionFrame) == MULTIPLANNER_TAB) then
		GarrisonMissionFrame_SelectTab(1); -- Missions
	end
end

local function onMissionFinished(onShow)
	local self = GarrisonMissionFrame;
	if ( self.MissionComplete:IsShown() ) then
		return;
	end

	if ( #self.MissionComplete.completeMissions > 0 and self:IsShown() ) then
		if ( PanelTemplates_GetSelectedTab(self) == MULTIPLANNER_TAB ) then
			GarrisonMissionFrame_SelectTab(1); -- Missions
		end
	end
end

function GarrisonPlanner_ExpandDetailsPane()
	local PlannerTab = GarrisonMissionFrame.PlannerTab
	local ScrollFrame = PlannerTab.listScroll
	local DetailsPane = PlannerTab.DetailsPane
	local ExpandButton = PlannerTab.DetailsExpandOrCollapseButton

	if ( DetailsPane.isCollapsed ) then
		DetailsPane.isCollapsed = false
		DetailsPane:Show()
		OPTIONS.setMultiPlannerShowDetailsPane(true)

		ScrollFrame:ClearAllPoints()
		ScrollFrame:SetPoint("TOPLEFT", PlannerTab.DetailsPane, "BOTTOMLEFT", 0, -6)
		ScrollFrame:SetPoint("BOTTOMRIGHT", -3, 8)

		ExpandButton:SetDisabledTexture("Interface\\Buttons\\UI-MinusButton-Disabled")
		ExpandButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
		ExpandButton:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-Down")

		PlannerMissionList_Update()
	end
end

function GarrisonPlanner_CollapseDetailsPane()
	local PlannerTab = GarrisonMissionFrame.PlannerTab
	local ScrollFrame = PlannerTab.listScroll
	local DetailsPane = PlannerTab.DetailsPane
	local CollpaseButton = PlannerTab.DetailsExpandOrCollapseButton

	if ( not DetailsPane.isCollapsed ) then
		DetailsPane.isCollapsed = true
		DetailsPane:Hide();
		OPTIONS.setMultiPlannerShowDetailsPane(false)

		ScrollFrame:ClearAllPoints()
		ScrollFrame:SetPoint("TOPLEFT", PlannerTab.DetailsPane, 0, -16)
		ScrollFrame:SetPoint("BOTTOMRIGHT", -3, 8)

		CollpaseButton:SetDisabledTexture("Interface\\Buttons\\UI-PlusButton-Disabled")
		CollpaseButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
		CollpaseButton:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down")

		PlannerMissionList_Update()
	end
end

local function disablePlannerTab()
	PanelTemplates_DisableTab(GarrisonMissionFrame, MULTIPLANNER_TAB)
end

local function enablePlannerTab()
	PanelTemplates_EnableTab(GarrisonMissionFrame, MULTIPLANNER_TAB)
end

local function initialize()
	local self = GarrisonMissionFrame.PlannerTab;
	self.listScroll.update = PlannerMissionList_Update;
	HybridScrollFrame_CreateButtons(self.listScroll, "MultiPlannerListButtonTemplate", 5, -4, nil, nil, nil, -3);
	PlannerMissionList_Update();


	GarrisonMissionFrame:HookScript("OnHide", clearAllParties);
	GarrisonMissionFrame:HookScript("OnHide", selectMissionsTab);

	for _,button in pairs(self.listScroll.buttons) do
		button:HookScript("OnClick", PlannerMissionList_Update);

		for i = 1, #button.Followers do
			button.Followers[i]:SetID(i); -- IDs don't seem to be inherited, so we set them manually
		end
	end

	-- Add Planner tab
	module.GarrisonMissionFrame_SelectTab = GarrisonMissionFrame_SelectTab
	GarrisonMissionFrame_SelectTab = selectTab

	-- Disable PlannerTab on CompleteDialog show
	local completeDialog = GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog
	completeDialog:HookScript("OnShow", disablePlannerTab);
	completeDialog:HookScript("OnHide", enablePlannerTab);

	-- Disable PlannerTab on MissionComplete show
	local missionComplete = GarrisonMissionFrame.MissionComplete
	missionComplete:HookScript("OnShow", disablePlannerTab);
	missionComplete:HookScript("OnHide", enablePlannerTab);

	-- Switch to MissionTab on GARRISON_MISSION_FINISHED event to display the CompleteDialog
	hooksecurefunc("GarrisonMissionFrame_CheckCompleteMissions", onMissionFinished)


	PanelTemplates_SetNumTabs(GarrisonMissionFrame, 3);
	PanelTemplates_UpdateTabs(GarrisonMissionFrame);

	module.dragndrop_init()

	-- Recalculate mission ready count
	GarrisonMissionFrame.PlannerTab:HookScript("OnShow", module.updateReadyMissionCount);

	-- MissionSelected
	Lib.RegisterCallback("multiplanner", "MISSION_SELECTED", PlannerMissionList_Update);
	Lib.RegisterCallback("multiplanner.details", "MISSION_SELECTED", updateDetailsPane);
	Lib.RegisterCallback("multiplanner.followerList", "MISSION_SELECTED", updateFollowerListForMission);

	-- MissionClosed
	Lib.RegisterCallback(Lib, "MissionClosed", PlannerMissionList_Update);

	-- Update Details pane for follower changes
	Lib.RegisterCallback("multiplanner.details", "FOLLOWER_ADDED_TO_MISSION", updateDetailsPaneOnFollowerChange)
	Lib.RegisterCallback("multiplanner.details", "FOLLOWER_REMOVED_FROM_MISSION", updateDetailsPaneOnFollowerChange)

	-- Show/Hide details pane according to preference
	if (OPTIONS.ShowMultiPlannerDetailsPane() ) then
		GarrisonPlanner_ExpandDetailsPane()
	else
		GarrisonPlanner_CollapseDetailsPane()
	end

	-- Filterable detailspane enemy mechanics
	local DetailsPane = GarrisonMissionFrame.PlannerTab.DetailsPane
	for e = 1, #DetailsPane.Enemies do
		local enemy = DetailsPane.Enemies[e]
		for m = 1, #enemy.Mechanics do
			enemy.Mechanics[m]:SetScript("OnClick", MissionMinder_FilterFollowersForCounter)
		end
	end

	-- Mark planner tab as beta
	GarrisonMissionFrame.Tab3:SetText("|cff00ff00*BETA*|r " .. GARRISON_PLANNER)

	module.setState(module.PLANNER)
end

local events = Lib.events
events:RegisterEvent("multiplanner", "ADDON_LOADED", initialize)
events:RegisterEvent("multiplanner", "GARRISON_MISSION_LIST_UPDATE", PlannerMissionList_Update);
